/*
**  Author          :   Aico Peter van Vuuren
**                      Tinbergen Institute Amsterdam
**                      Vrije Universiteit, Department of Economics
**  Final version   :   27 october 1998
**
**  This program computes the marginal tax rate of individuals at
**  a given wage level. Data is used from OECD figures.
*/

fiefra(const x, const adFunc)
{
    decl tau = transfra(x);
    adFunc[0] = (1-tau).*x - g_b1;
	return 1;
}

fiefra1(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transfra(x);
    adFunc[0] = (1-tau[0]).*x[0] - g_b1[0];
	return 1;
}

fiefra2(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transfra(x);
    adFunc[0] = (1-tau[1]).*x[1] - g_b1[1];
	return 1;
}

fiefra3(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transfra(x);
    adFunc[0] = (1-tau[2]).*x[2] - g_b1[2];
	return 1;
}

fieger(const x, const adFunc)
{
    decl tau = transd(x);
    adFunc[0] = (1-tau).*x - g_b1;
	return 1;
}

fieger1(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transd(x);
    adFunc[0] = (1-tau[0]).*x[0] - g_b1[0];
	return 1;
}

fieger2(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transd(x);
    adFunc[0] = (1-tau[1]).*x[1] - g_b1[1];
	return 1;
}

fieger3(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transd(x);
    adFunc[0] = (1-tau[2]).*x[2] - g_b1[2];
	return 1;
}

fiened(const x, const adFunc)
{
    decl tau = transned(x);
    adFunc[0] = (1-tau).*x - g_b1;
	return 1;
}

fiened1(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transned(x);
    adFunc[0] = (1-tau[0]).*x[0] - g_b1[0];
	return 1;
}

fiened2(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transned(x);
    adFunc[0] = (1-tau[1]).*x[1] - g_b1[1];
	return 1;
}

fiened3(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transned(x);
    adFunc[0] = (1-tau[2]).*x[2] - g_b1[2];
	return 1;
}

fieuk(const x, const adFunc)
{
    decl tau = transuk(x);
    adFunc[0] = (1-tau).*x - g_b1;
	return 1;
}

fieuk1(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transuk(x);
    adFunc[0] = (1-tau[0]).*x[0] - g_b1[0];
	return 1;
}

fieuk2(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transuk(x);
    adFunc[0] = (1-tau[1]).*x[1] - g_b1[1];
	return 1;
}

fieuk3(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transuk(x);
    adFunc[0] = (1-tau[2]).*x[2] - g_b1[2];
	return 1;
}

berb(const b, fie, fie1, fie2, fie3)
{
	decl gradp, adfunc, x, hulp1, hulp2, hulp3;
    x = vec(b.*0.1);

    g_b1 = b;
	fie(x,&adfunc);

	while (fabs(adfunc' * adfunc) > 0.0001)
	{
		Num1Derivative(fie1, vec(x), &hulp1);
		Num1Derivative(fie2, vec(x), &hulp2);
		Num1Derivative(fie3, vec(x), &hulp3);
		gradp = hulp1[0]|hulp2[1]|hulp3[2];
		fie(x,&adfunc);
		x = x - adfunc./gradp;
    }
	
    return x;
}

transned(const x)
{
    decl perc, brac, tax1, tax2, tax3, tax, t1, t2, t3,
            rent, mininc, minrent, avginc, avgrent, t, ti, hous, hous1,
            hous2, sc, franchise, sub1, sub2, sub3, sub4, allow1, allow2,
            allow3, allow4, ts, fc;

    decl wage = x .* 12. .* g_dollar;

    /* earned income tax credit */

    if (g_eitc == 1)
	{
        fc = berfc1(wage[0].*(1 ./g_dollar));
        fc = fc|berfc2(wage[0].*(1 ./g_dollar));
        fc = fc|berfc2(wage[0].*(1 ./g_dollar));
    }
	else
	{
        fc = zeros(3,1);
    }

    /* social security contributions */

    sc =  (0.01+0.0255) .* wage .* (wage .< 74360) +
            (0.01+0.0255) .* 74360 .* (wage .> 74360);
    franchise = 25740;
    sc = sc + (wage .> franchise) .* ((wage-franchise).*0.094.*
            (wage .<= 74360) + (74360-franchise).*0.094.* (wage .> 74360));

    /* allowances */

    sub1 = wage - sc;

    allow1 = sub1 .* 0.08 .* ((sub1 .* 0.08 .<= 2139) .&& (sub1 .* 0.08)
            .>= 237) + 2137 .* (sub1 .* 0.08 .> 2139) +
                237 .* (sub1 .* 0.08 .< 237);

    sub2 = sub1 - allow1;
    allow2  = sub2 .* 0.1175 .* (sub2 .<= 76350) + sub2 .* (sub2 .> 76350)
                .* 0.1175;

    sub3 = sub1 + allow2;
    allow3 = sub3 .* 0.08 .* ((sub3 .* 0.08 .<= 2139) .&& (sub3 .* 0.08)
            .>= 237) + 2139 .* (sub3 .* 0.08 .> 2139) +
                237 .* (sub3 .* 0.08 .< 237);
    sub4 = sub3 - allow3;
    allow4 = 6074|(6074+4860)|(2 .*6074);
    ts = sub4 - allow4;

    /* taxes */

    perc = (14.55+1.8+6.3+6.15+8.85)|50|60;
    perc = perc ./ 100;
    brac = 44450|88696;

    t1 = ts .< brac[0];
    t2 = (ts .< brac[1]) .&& (ts .> brac[0]);
    t3 = (ts .> brac[1]);

    tax1 = ts .* perc[0];
    tax2 = (t2+t3) .* (ts - brac[0]) .* (perc[1]-perc[0]);
    tax3 = t3 .* (perc[2]-perc[1]) .* (ts - brac[1]);
    tax = tax1+tax2+tax3;
    tax[0] = tax[0]+110;
    tax[1:2] = tax[1:2]+2 .*110;

    /*  housing benefits */

    ti = ts;

    avgrent = 7490;
    avginc = 39500;
    minrent = 3820;
    mininc = 29200;
    rent = 4900;
    t = ((ti-mininc)*(avgrent/avginc)+(avginc-ti)*(minrent/mininc))
                ./(avginc-mininc);
    t = ti.*t;
    hous1 = max((rent - (t+0.05*rent)) .* (ti.<42000)|0);
    ti = wage[0]-tax[0];

    avginc = 28400;
    minrent = 3820;
    mininc = 22400;
    rent = 4900;
    t = ((ti-mininc)*(avgrent/avginc)+(avginc-ti)*(minrent/mininc))
                ./(avginc-mininc);
    t = ti.*t;
    hous2 = max((rent - (t+0.05*rent)) .* (ti.<30000)|0);
    hous = hous2|hous1|hous1;

    /* family benefits */

    tax[1:2] = tax[1:2] - 2954;

    tax = tax + sc - hous;
    tax = tax - fc;
    return tax./wage;
}

transuk(const x)
{
    decl allow, perc, wagedes, brac, tax1, tax2, tax3, tax, t1, t2, t3,
        fam, hous,sc, fc, wage;

    wage = x .* 12 .* (g_dollar./g_pound);

    if (g_eitc == 0)
	{
        fc = berfc2(wage .*(g_pound./g_dollar)).*(0|1|1);
    }

    sc = 0.02 .* (wage - 58) .* (wage .> 58) + 0.1 .* (wage - 116) .*
            ((wage .> 116) .&& (wage .< 440)) + 0.1 .* (440 - 116) .*
            (wage .> 440);

    allow = 3525|(3525+258)|(3525+258);
    wagedes = (wage-allow) .* (wage .> allow);

    perc = 20|25|40;
    perc = perc ./ 100;
    brac = 3200|24300;

    t1 = wagedes .< brac[0];
    t2 = (wagedes .< brac[1]) .&& (wagedes .> brac[0]);
    t3 = (wagedes .> brac[1]);

    tax1 = wagedes .* perc[0];
    tax2 = (t2+t3) .* (wagedes - brac[0]) .* (perc[1]-perc[0]);
    tax3 = t3 .* (perc[2]-perc[1]) .* (wagedes - brac[1]);
    tax = tax1+tax2+tax3;
    fam = (10.4+8.45)*52;
    tax[1:2] = tax[1:2] - fam;
    hous = 3047 - 0.65.*wage;
    hous = hous .* (hous .> 0);
    tax = tax - hous;

    if (g_eitc .== 0)
        tax = tax - fc;
		
    tax = tax + 592.8 + sc;
	
    return tax./wage;
}

transfra(const x)
{
    decl allow, perc, wagedes, brac, tax1, tax2, tax3, tax
        , tax4, tax5, tax6, tax7, sc, allow1, allow2, k, r, cm, n, lo, l, c,
        hous, t1, t2, t3, t4, t5, t6, t7, b1, b2, b3, b4, b5, weight,
        sc1, sc2, sc3, sc4, sc5, sc6, split, fc, wage;

    wage = x .* 12 .* (g_dollar./g_frank);


    if (g_eitc == 1)
	{
        fc = berfc1(wage[0].*(g_frank./g_dollar));
        fc = fc|berfc2(wage[0].*(g_frank./g_dollar));
        fc = fc|berfc2(wage[0].*(g_frank./g_dollar));
	}
    else
	{
        fc = zeros(3,1);
    }

    /* social security contributions */

    sc1 = (0.06 .* wage) .* (wage .<= 155940) + (0.0655 .* 155940) .*
                (wage .> 155940);
    sc2 = 0.068 .* wage;
    sc3 = (0.0322 .* wage) .* (wage .<= 155940) + (0.0386 .* wage) .*
                ((wage .> 155940) .&& (wage .<= 623760)) + (0.0386 .* 623760)
                .* (wage .> 623760);
    sc4 = (0.02 .* wage) .* (wage .<= 467820) + (0.02 .* wage) .*
            (wage .> 467820);
    sc5 = 0.001 .* wage;
    sc = sc1+sc2+sc3+sc4+sc5;

    /* tax allowences */

    allow1 = 0.1*(wage-sc);
    allow1 = 2230 .* (allow1.<2230) + ((allow1 .> 2230) .&& (allow1 .< 74590))
                .* allow1 + 74590 .* (allow1 .> 74590);
    allow2 = 0.2*(wage-sc-allow1);
    allow2 = allow2 .* (allow2 .< 13600) + allow2 .* (allow2 .>= 13600);
    wagedes = wage - allow1 - allow2 - sc - g_eitc;

    perc = 0|12|25|35|45|50|56.8;
    perc = perc ./ 100;

    brac = 22610|49440|87020|140900|229260|282730;

    /* housing benefits */

    r = wagedes[0];

    cm = 101184;

    n = 1.2;
    k = 0.9 - (r/(cm.*n));
    c = 282 .* 12;
    l = 12 .* 1266;
    b1 = r .< 8076;
    b2 = (r .> 8067) && (r .< 11606);
    b3 = (r .> 11606) && (r .< 14908);
    b4 = (r .> 14908) && (r .< 23213);
    b5 = (r .> 23213);
    lo = (b1 .* 0 + b2 .* 0.03 + b3 .* 0.26 + b4 .* 0.29 + b5 .* 0.41) .* l;
    hous = k .* (l+c-lo);

    r = wagedes[1:2];
    n = 3;
    k = 0.9 - (r/(cm.*n));
    l = 12 .* 1738;

    c = 404 .* 12;
    b1 = r .< 20166;
    b2 = (r .> 20166) .&& (r .< 29013);
    b3 = (r .> 29013) .&& (r .< 37269);
    b4 = (r .> 37269) .&& (r .< 58032);
    b5 = (r .> 58032);
    lo = (b1 .* 0 + b2 .* 0.03 + b3 .* 0.26 + b4 .* 0.29 + b5 .* 0.41) .* l;

    hous = hous|(k .* (l+c-lo));
    hous = hous .* (hous .> 0) + 0 .* (hous .<= 0);
    weight = 1|2.5|3;
    split = wagedes ./ (1|2|3);

    t1 = split .< brac[0];
    t2 = (split .< brac[1]) .&& (split .> brac[0]);
    t3 = (split .< brac[2]) .&& (split .> brac[1]);
    t4 = (split .< brac[3]) .&& (split .> brac[2]);
    t5 = (split .< brac[4]) .&& (split .> brac[3]);
    t6 = (split .< brac[5]) .&& (split .> brac[4]);
    t7 = split .> brac[5];

    tax1 = 0;
    tax2 = t2 .* (split .* perc[1] - 2713.2);
    tax3 = t3 .* (split .* perc[2] - 9140.4);
    tax4 = t4 .* (split .* perc[3] - 17842.4);
    tax5 = t5 .* (split .* perc[4] - 31932.4);
    tax6 = t6 .* (split .* perc[5] - 43395.4);
    tax7 = t7 .* (split .* perc[6] - 62621.04);

    tax = tax1+tax2+tax3+tax4+tax5+tax6+tax7;
    tax = tax .* (1|2|3);
    tax = tax+sc;
    tax = tax+wage.*0.024.*0.95 + 1310;

    tax[1:2] = tax[1:2] - 7980;

    tax = tax - hous;
    tax = tax - fc;

    return tax./wage;
}

transd(const vx)
{
    decl sc, allow, scallow, hulp, x, y, t, tax, ni, fam, ceil1,
        rent, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13,
        p14, p15, p16, lop, hop, sv1, sv2, sv3, sv4, til, acc, own, hous,
        hulp1, t1, fc, wage;

    wage = vx .* 12 .* (g_dollar./g_mark);

    /* earned income tax credit */

    if (g_eitc == 1)
	{
        fc = berfc1(wage[0].*(g_mark./g_dollar));
        fc = fc|berfc2(wage[0].*(g_mark./g_dollar));
        fc = fc|berfc2(wage[0].*(g_mark./g_dollar));
    }
	else
	{
        fc = zeros(3,1);
    }

    /* social security contributions */

    sc = 0.093 .* wage .* (wage .< 93600) + 0.093 .* 93600 .* (wage .>= 93600);
    sc = sc + 0.035 .* wage .* (wage .< 93600) + 0.035 .* 93600
                .* (wage .>= 93600);
    sc = sc + 0.0685 .* wage .* (wage .< 70200) + 0.0685 .* 70200
                .* (wage .>= 70200);
    sc = sc + 0.005 .* wage .* (wage .< 70200) + 0.005 .* 70200
                .* (wage .>= 70200);

    /* allowances */

    allow = 0|(2*4104)|(2*4104);
    hulp = 0|(2*2052+5616)|0;
    allow = allow + hulp;
    allow = allow + 2000;
    scallow = (6000|6000|12000) - 0.16 .* wage;
    scallow = scallow.* (scallow .> 0) + 0 .* (scallow .<= 0);
    scallow = scallow + (sc - scallow) .* ((sc - scallow .> 0) .&&
                ((sc - scallow) .< 2610|2610|5220)) +
                        (2610|2610|5220) .* ((sc - scallow) .>= 2610|2610|5220);
    scallow = scallow + (sc - scallow) .* ((sc - scallow .> 0) .&&
                (sc - scallow .< 1305|1305|2610)) +
                        (1305|1305|2610) .* ((sc - scallow) .>= 1305|1305|2610);
    allow = allow + scallow;

    x = (wage - allow) ./ (1|1|2);
    y = (x-8100)./10000;
    t = 0 .* (x .<= 5616);
    t = t + (0.19 .* x - 1067) .* ((5616 .< x) .&&
                (x .<= 8153));
    t = t + ((151.94.*y + 1900).*y + 472) .*
                ((x .> 8153) .&& (x .<= 120042));
    t = t + (0.53 .*x - 22842) .* (x .> 120042);
    hulp1 = ((t .> (t-1135).*2.5) .&& ((t-1135) .> 0));
    t = t .* (t .> 1135);
    t = (t-1135).*2.5 .* hulp1 + (1-hulp1).*t;
    t = t .* (1|1|2);
    tax = t + sc;

    /* family benefits */

    ni = wage - tax;
    ceil1 = (19000|19000|26600) + 2 .* 9200;
    fam = 840 + 1560 - 0.5 .* (wage-ceil1) .* ((wage - ceil1 .> 0) .&&
            (wage-ceil1 .<= 1560)) - 840 .* (wage - ceil1 .> 1560);
    tax[1:2] = tax[1:2] - fam[1:2];

    /* housing benefits */

    rent = 400|800|800;
    p1 = 45|65|65;
    p2 = 630|1130|1130;
    p3 = 230|490|490;
    p4 = 1410|3250|3250;
    p5 = 621.261|1405.9|1405.9;
    p6 = 0.798|1.030|1.030;
    p7 = 319.361|548.9|548.9;
    p8 = 89.910|159.840|159.840;
    p9 = 6.21E-4|3.03E-4|3.03E-4;
    p10 = 5.23E-4|3.67E-4|2.43E-4;
    p11 = 0.119|0.107|0.107;
    p12 = 1.8E-5|0|0;
    p13 = 3.5E-5|1E-6|1E-6;
    p14 = 1.2E-4|6.3E-5|6.3E-5;
    p15 = 2.45E-7|0|0;
    p16 = 523.906|686.67|686.67;

    acc = 0.7 .* (wage./12 - 167);
    til = p5 + p6 .* (rent-p7);
    sv1 = p10;
    sv2 = p11 .* sv1.*(rent-p8);
    sv3 = (p12-p13) .* til +sv2;
    sv4 = p14 - rent .* p15;
    hop = (acc .<= til) .* ((p12.*acc+sv2).*acc) +
            (acc .> til) .* ((p13.*acc+sv3).*acc);
    lop = (til .>= p16) .* ((sv4.*acc+p16.*(p12-sv4)+sv2).*acc)
                + (til .< p16) .* ((sv4.*acc+p16.*(p13-sv4)).*acc);
    own = (acc .> p16) .* hop + (acc .<= p16) .* lop;
    hous = rent .* (acc .< p3) + (rent - own) .* ((acc .>= p3)
            .&& (acc .< p4));

    tax = tax - hous;
    tax = tax - fc;
    return tax./wage;
}
 
berfc3(const wage)
{
    if (wage < g_frb[2])
        return g_sr[2].*wage;
    if (wage < g_fre[2])
        return g_mc[1];
	if (wage < g_co[2])
        return g_mc[2] - g_por[2] .* (wage - g_fre[2]);
   
    return 0;
}


berfc2(wage)
{
    decl wage2, sc, allow, wagedes, tax1, tax2, tax3, t1, t2, t3, perc,
        min1, min2, result, credit, brac, tax, net;

    wage2 = wage.*(g_dollar./g_pound);

    sc = 0.02 .* (wage2 - 58) .* (wage2 .> 58) + 0.1 .* (wage2 - 116) .*
            ((wage2 .> 116) .&& (wage2 .< 440)) + 0.1 .* (440 - 116) .*
            (wage2 .> 440);

    allow = 3525+258;
    wagedes = (wage2-allow) .* (wage2 .> allow);

    perc = 20|25|40;
    perc = perc ./ 100;
    brac = 3200|24300;

    t1 = wagedes .< brac[0];
    t2 = (wagedes .< brac[1]) .&& (wagedes .> brac[0]);
    t3 = (wagedes .> brac[1]);

    tax1 = wagedes .* perc[0];
    tax2 = (t2+t3) .* (wagedes - brac[0]) .* (perc[1]-perc[0]);
    tax3 = t3 .* (perc[2]-perc[1]) .* (wagedes - brac[1]);
    tax = tax1+tax2+tax3;

    tax = tax;
    net = (wage2-tax)./52;	   // net wage per week in pounds
	println(net);
    credit = (11.4+18.9+45.1+10);
    min1 = 73;
    min2 = 0.5;
    result = credit - 0.7.*(net-min1).*((net-min1).>0);
	println(credit);
    result = result.*(result.>=min2).*52 + 0 .*(result.<min2);
    return result .* (g_pound ./ g_dollar);
}

berfc1(const wage)
{
    return 0;
}

init1()
{
    g_eitc = 1;
}

init_tax()
{
    g_eitc = 0;
    g_pound = 3.409;
    g_mark = 1.126744;
    g_frank = 0.335953;
    g_dollar = 2.0285;

    g_frb = 4000|8640;
    g_fre = 5130|11290;
    g_co = 9230|26613;
    g_mc = 314|3110;
    g_sr = 0.0765|0.36;
    g_por = 0.0765|0.2022;

    g_frb = g_frb.*(g_pound./g_dollar);
    g_fre = g_fre.*(g_pound./g_dollar);
    g_co = g_co.*(g_pound./g_dollar);
    g_mc = g_mc.*(g_pound./g_dollar);
}

