/*
**  Author          :   Aico Peter van Vuuren
**                      Tinbergen Institute Amsterdam
**                      Vrije Universiteit, Department of Economics
**  Final version   :   27 october 1998
**
**  This program computes the marginal tax rate of individuals at
**  a given wage level. Data is used from OECD figures.
*/

fieuk(const x, const adFunc)
{
    decl tau = transuk(x);
	decl dh = berfc2(x).*(0|1|1);
	
    adFunc[0] = (1-tau).*x + (1 - g_eitc) .* dh - g_b1;
	return 1;
}

fieuk1(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transuk(x);
    adFunc[0] = (1-tau[0]).*x[0] - g_b1[0];
	return 1;
}

fieuk2(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transuk(x);
	decl dh = berfc2(x);
    adFunc[0] = (1-tau[1]).*x[1] + (1 - g_eitc) .* dh[1] - g_b1[1];
	return 1;
}

fieuk3(const x, const adFunc, const avScore, const mHess)
{
    decl tau = transuk(x);
	decl dh = berfc2(x);
    adFunc[0] = (1-tau[2]).*x[2] + (1 - g_eitc) .* dh[2] - g_b1[2];
	return 1;
}

berb(const b, fie, fie1, fie2, fie3)
{
	decl gradp, adfunc, x, hulp1, hulp2, hulp3;
    x = vec(b.*0.1);

    g_b1 = b;
	fie(x,&adfunc);

	while (fabs(adfunc' * adfunc) > 0.0001)
	{
		Num1Derivative(fie1, vec(x), &hulp1);
		Num1Derivative(fie2, vec(x), &hulp2);
		Num1Derivative(fie3, vec(x), &hulp3);
		gradp = hulp1[0]|hulp2[1]|hulp3[2];
		fie(x,&adfunc);

		x = x - adfunc./gradp;
    }
	
    return x;
}

transuk(const x)
{
    decl allow, perc, wagedes, brac, tax1, tax2, tax3, tax, t1, t2, t3,
        fam, hous,sc, fc, wage;

    wage = x .* 12;

    sc = 0.02 .* (wage - 58) .* (wage .> 58) + 0.1 .* (wage - 116) .*
            ((wage .> 116) .&& (wage .< 440)) + 0.1 .* (440 - 116) .*
            (wage .> 440);

    allow = 3525|(3525+258)|(3525+258);
    wagedes = (wage-allow) .* (wage .> allow);

    perc = 20|25|40;
    perc = perc ./ 100;
    brac = 3200|24300;

    t1 = wagedes .< brac[0];
    t2 = (wagedes .< brac[1]) .&& (wagedes .> brac[0]);
    t3 = (wagedes .> brac[1]);

    tax1 = wagedes .* perc[0];
    tax2 = (t2+t3) .* (wagedes - brac[0]) .* (perc[1]-perc[0]);
    tax3 = t3 .* (perc[2]-perc[1]) .* (wagedes - brac[1]);
    tax = tax1+tax2+tax3;
    fam = (10.4+8.45)*52;
    tax[1:2] = tax[1:2] - fam;
    hous = 3047 - 0.65.*wage;
    hous = hous .* (hous .> 0);
    tax = tax - hous;
		
    tax = tax + 592.8 + sc;
	
    return tax./wage;
}

berfc3(const wage)
{
    if (wage < g_frb[2])
        return g_sr[2].*wage;
    if (wage < g_fre[2])
        return g_mc[1];
	if (wage < g_co[2])
        return g_mc[2] - g_por[2] .* (wage - g_fre[2]);
   
    return 0;
}


berfc2(wage)
{
    decl wage2, sc, allow, wagedes, tax1, tax2, tax3, t1, t2, t3, perc,
        min1, min2, result, credit, brac, tax, net;

	wage2 = wage .* 12;

    sc = 0.02 .* (wage2 - 58) .* (wage2 .> 58) + 0.1 .* (wage2 - 116) .*
            ((wage2 .> 116) .&& (wage2 .< 440)) + 0.1 .* (440 - 116) .*
            (wage2 .> 440);

    allow = 3525+258;
    wagedes = (wage2-allow) .* (wage2 .> allow);

    perc = 20|25|40;
    perc = perc ./ 100;
    brac = 3200|24300;

    t1 = wagedes .< brac[0];
    t2 = (wagedes .< brac[1]) .&& (wagedes .> brac[0]);
    t3 = (wagedes .> brac[1]);

    tax1 = wagedes .* perc[0];
    tax2 = (t2+t3) .* (wagedes - brac[0]) .* (perc[1]-perc[0]);
    tax3 = t3 .* (perc[2]-perc[1]) .* (wagedes - brac[1]);
    tax = tax1+tax2+tax3;

    tax = tax;
    net = (wage2-tax)./52;	   // net wage per week in pounds
    credit = (11.4+18.9+45.1+10);
    min1 = 73;
    min2 = 0.5;
	
    result = credit - 0.7.*(net-min1).*((net-min1).>0);
    result = result.*(result.>=min2).*(52 ./ 12) + 0 .*(result.<min2);
    return result;
}

berfc1(const wage)
{
    return 0;
}

init1()
{
    g_eitc = 1;
}

init_tax()
{
    g_eitc = 0;
    g_pound = 3.409;
    g_mark = 1.126744;
    g_frank = 0.335953;
    g_dollar = 2.0285;

    g_frb = 4000|8640;
    g_fre = 5130|11290;
    g_co = 9230|26613;
    g_mc = 314|3110;
    g_sr = 0.0765|0.36;
    g_por = 0.0765|0.2022;

    g_frb = g_frb.*(g_pound./g_dollar);
    g_fre = g_fre.*(g_pound./g_dollar);
    g_co = g_co.*(g_pound./g_dollar);
    g_mc = g_mc.*(g_pound./g_dollar);
}

